/*
 *
 * (c) Copyright 1990 by William R. Pringle - all rights reserved
 *
 *				**** BLD KEY ****
 *
 * EMACS_MODES: c, !fill, tabstop=4, !lnumb
 *
 * This program reads a music data file and produces both an index
 *	file as well as a key file.
 *
 * Input parameters is list of input file names
 *
 * Input file name should have a suffix of .dat
 * Output key file will have suffix .kxx
 * Output index file will have suffix .ixx
 *	where xx is field number (0 - artist, 1 - title)
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include "cd.h"

main(argc,argv)
int	argc;
char **argv;
{
	FILE	*open_out(),		/* routine to open output files */
			*infile,			/* input file for master file */
			*keyart,			/* artist key file */
			*keytit,			/* title key file */
			*keycut;			/* cut key file */

	char	fname[BUFSIZ];		/* master file name */
	char	datname[BUFSIZ];	/* data file name */
	char	dirname[BUFSIZ];	/* directory name */
	char	buf[BUFSIZ];		/* input buffer */
	char	*p;					/* for parsing */
	int		file_count;			/* file counter */
	long	rec_count;			/* record counter */
	int		nrec;				/* records read for each call */
	char	*flds[MASTFLDS];	/* fields in master record */
	int		nflds;				/* how many fields were found */
	int		ftype;				/* file type (CD, cassette, record)

	debugsw = 1;				/* turn on debugging */
	Prog = argv[0];				/* my name */

		/* look for directory name */
#ifdef __MSDOS__
	p = strrchr(Prog,'\\');	/* MS-DOS directory */
	_fmode = O_BINARY;		/* set default file type as well */
#else
	p = strrchr(Prog,'/');
#endif
	if (p != NULL) Prog = p+1;	/* actual name */
	argc--;			/* shift my name out */
	argv++;			/* of argument list */

	while (argv[0][0] == '-')
	{		/* option */
		switch ((int) argv[0][1])
		{
			case 'd':	/* turn on debugging */
				if (argv[0][2] == EOS)
				{
					debugsw = atoi(argv[1]);	/* get value */
					argc -= 2;					/* shift out ... */
					argv += 2;					/* flag and value */
				} else
				{
					debugsw = atoi(argv[0]+2);	/* value follows flag */
					argc--;
					argv++;
				}
				break;

			case 'h':	/* help */
				help();
				exit(0);

			default:	/* invalid option */
				fprintf(stderr,"Invalid option (%s)\n", argv[0]);
				usage();
				exit(1);
		}
	}

	if (argc == 0)
	{
		fprintf(stderr,"No file names given\n");
		usage();
		exit(1);
	}

	while (argc > 0)
	{
		printf("Processing master file %s\n", argv[0]);

		strcpy(fname,argv[0]);			/* copy name of file */
		if ((infile=fopen(fname,"r")) == NULL)
		{
			fprintf(stderr,"%s: Unable to open master file %s\n", Prog, fname);
			argc--;
			argv++;
			continue;
		}

		p = strrchr(fname,'.');		/* find suffix (if any) */
		if (p == NULL) p = fname + strlen(fname);	/* if not, end of name */

		*p = EOS;					/* get just base name */
		keyart = open_out(fname,".kar",0);		/* artist key file */
		keytit = open_out(fname,".kti",0);		/* title key file */
		keycut = open_out(fname,".kcu",0);		/* cut key file */
		if (keyart == NULL || keytit == NULL || keycut == NULL)
		{
			printf("Unable to open key file(s) for output.\n");
			exit(1);
		}

		strcpy(dirname,fname);					/* get full path name */
		p = strrchr(dirname,'/');				/* find end of path */
		if (p == NULL) p = strrchr(dirname,'\\');	/* maybe this slash */
		if (p == NULL) p = dirname;				/* no path name */
		else p++;								/* point past slash */
		*p = EOS;								/* terminate dir name */

		file_count = 0;
		rec_count = 0L;

		while (fgets(buf,BUFSIZ,infile) != NULL)
		{		/* read all data files in database */
				/* clean up file name (remove CR-LF and ^Z) */
			p = clean_up(buf);
			if (*p == EOS) continue;
			if ((nflds = split_flds(p,flds,'|',0,MASTFLDS)) != MASTFLDS)
			{		/* wrong number of fields */
				fprintf(stderr,"%s: %d fields read from master file %s\n",
								Prog, nflds, fname);
				fprintf(stderr,"\t(instead of %d) >>%s<<\n",
							MASTFLDS, buf);
				continue;
			}

			printf("\tProcessing data file %s ... ", flds[0]);
			if (*(flds[0]) != '/' && *(flds[0]) != '\\')
			{		/* no path name, assume same directory */
				strcpy(datname,dirname);		/* same directory */
				strcat(datname,flds[0]);		/* and this name */
			} else strcpy(datname,flds[0]);		/* full path name */


			ftype=find_ftype(flds[1]);
			switch((char) ftype) {
				case FTYPE_CD:		/* CD */
					nrec = find_cds(datname,file_count,keyart,keytit,keycut);	break;
				case FTYPE_CASS:	/* cassette tape */
					nrec = find_cass(datname,file_count,keyart,keytit,keycut);	break;
				case FTYPE_REC:		/* record */
					nrec = find_rec(datname,file_count,keyart,keytit,keycut);	break;
				default:			/* oops! */
					fprintf(stderr,
						"Invalid type of file (%s) for %s in master file %s\n",
							flds[1], flds[0], fname);
					continue;
			}

			printf("%d entries found\n", nrec);
			file_count++;		/* next file */
			rec_count += nrec;
		}

		fprintf(stderr,"%s: Processing complete, %d files and %ld albums found.\n",
					Prog, file_count, rec_count);

			/* record zero says how many records in file */
		fprintf(keyart,KEYFMT,"",0,0,rec_count,rec_count);
		fprintf(keytit,KEYFMT,"",0,0,rec_count,rec_count);
		tot_cuts(keycut);		/* write total number of cuts */

			/* close all files */
		fclose(infile);		/* master input file */
		fclose(keyart);
		fclose(keytit);
		fclose(keycut);

		argc--;
		argv++;
	}
	return 0;
}

usage()
{
	fprintf(stderr,"Usage: %s [ -d val ] files\n", Prog);
}

help()
{
	fprintf(stderr,"Help not available yet\n");
}
