/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** BLD TBL ****
 *
 * EMACS_MODES: c, !fill, tabstop=4, !lnumb
 *
 * This program builds a tbl input file for albums from a master file
 * The master file contains names of data files, which in turn contain
 *	the names of albums.
 *
 * This program extracts the album lines which can be used as input
 *	to tbl for printing
 *
 */

#include <stdio.h>
#include <string.h>
#include "cd.h"

main(argc,argv)
int argc;
char **argv;
{
	char	data_name[BUFSIZ];		/* data file name */
	FILE	*mfile;					/* master file pointer */
	FILE	*dfile;					/* data file name */
	char	*flds[MASTFLDS];		/* fields of master file */
	int		nflds;					/* number of fields found */
	char	*cd_flds[DATFLDS];		/* fields in data file */
	int		ncd_flds;				/* number of fields found */
	int		art_flds[ARTFLDS];		/* artist fields */
	int		nart_flds;				/* number of fields found */

	char	buf[BUFSIZ];			/* input buffer */
	char	*p;						/* temp ptr variable */
	long	loc;					/* location of album line */

	int		i;						/* for looping */

	Prog = argv[0];					/* my name */

		/* strip full path name if there */
#ifdef __MSDOS__
	p = strrchr(Prog,'\\');			/* Mess-Dos */
#else
	p = strrchr(Prog,'/');			/* Unix */
#endif

	if (p != NULL) Prog = p+1;		/* just executable name */

	argc--;			/* shift executable name */
	argv++;			/* out of range */

	while (argc > 0)
	{		/* should be file names */
		if ((mfile = fopen(argv[0],"r")) == NULL)
		{		/* error during open */
			fprintf(stderr,"%s: Can't open master file %s\n", Prog, argv[0]);
			exit(1);
		}

		while (fgets(data_name,BUFSIZ,mfile) != NULL)
		{		/* read a record, clean up file name */
			for (p=data_name; *p != EOS && *p != '\n' && *p != '\r'; p++);
			*p = EOS;
			if (*data_name == EOS) continue;

			if ((nflds=split_flds(data_name,flds,'|',0,MASTFLDS)) != MASTFLDS)
			{		/* wrong number of fields */
				fprintf(stderr,"%s: %d fields read from master file %s\n",
								Prog, nflds, argv[0]);
				fprintf(stderr,"\t(instead of %d) >>%s<<\n",
							MASTFLDS, buf);
				continue;
			}

			if ((dfile = fopen(flds[0],"r")) == NULL)
			{		/* error during open of data file */
				fprintf(stderr,"%s: Can't open data file %s\n",
							Prog, data_name);
				continue;
			}

					/* data file open, look for album records */

			while ((loc = find_cd(dfile,buf)) >= 0L)
			{		/* read a line -- clean it up */
				if (*buf == EOS) continue;	/* empty line */

				if ((ncd_flds=split_flds(buf,cd_flds,'|',0,DATFLDS)) != DATFLDS)
				{
					fprintf(stderr,"%s: %d fields found in %s\n",
						Prog, ncd_flds, argv[0]);
					fprintf(stderr,"\t(instead of %d) >>%s<<\n",
						DATFLDS, cd_flds[0]);
					continue;
				}

				nart_flds = split_flds(cd_flds[DAT_ART],art_flds,'~',0,ARTFLDS);
				for (i=0; i < nart_flds; i++)
				{		/* loop thru artist fields */
					printf("%s|%s|%s|%s|%s\n", art_flds[i], cd_flds[1],
							cd_flds[2], cd_flds[3], cd_flds[4]);
				}
			}
			fclose(dfile);
		}
		fclose(mfile);
		argc--;
		argv++;
	}
}
