#!/usr/local/pub/bin/gawk -f
# this script checks albums files

BEGIN {
	FS = "|"

	# positional parameters
	
	# album fields

	ALB_ART		= 1		# artist name
	ALB_TIT		= 2		# album title
	ALB_YR		= 3		# publication date
	ALB_LBL		= 4		# label name
	ALB_CAT		= 5		# catalogue number

	# cut record
	
	CUT_TIT		= 1		# cut title
	CUT_TIME	= 2		# cut time
	CUT_ART		= 3		# cut artist (optional)

	LOWER = "abcdefghijklmnopqrstuvwxyz"
	UPPER = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	NOTROMAN = "ABEFGHJKNOPQRSTUWYZ"
}

FILENAME != OLDFILE {	# new file name
	print "Processing file " FILENAME " ..."
	OLDFILE = FILENAME
	NR = 1
}

{		# for every record
	if (length(DEBUGSW)>0) printf "Line %d >>%s<<", NR, $0

	if (length($0) < 2)
	{
		if (length(DEBUGSW)>0) printf "Empty Line\n"
		AlbTit = AlbArt = ""
	} else if (substr($0,1,1) == "\t")
	{		# line is a cut - check it out
		if (length(DEBUGSW)>0) printf "\tCut line\n"

		Cuts++
		Cut = substr($CUT_TIT,2)
		ChkCap(Cut)

		if (NF > 3)
		{		# too many fields
			printf "--Line %d: Too many fields for a cut >>%s<<\n", NR, $0
		} else if (NF > 1)
		{		# check time field
			if (length($CUT_TIME) == 0) nop = 1	# do nothing
			else if (length($CUT_TIME) > 5 || length($CUT_TIME) < 4)
			{		# wrong length
				printf "--Line %d: Wrong length time field ", NR
				printf "(%s - %s) >>%s<<\n", AlbArt, AlbTit, $0
			} else
			{
				Prob = 0
				 for (i = 1; i <= length($CUT_TIME); i++)
				{
					if (index("0123456789:",substr($CUT_TIME,i,1)) == 0)
						Prob = i
				}
				if (Prob > 0)
				{		# invalid date
					printf "--Line %d: Invalid cut time (char %d) ", NR, Prob
					printf "(%s -	%s) >>%s<<\n", AlbArt, AlbTit, $0
				}
			}
		} else
		{
			printf "--Line %d: Missing time ", NR
			printf " (%s - %s) >>%s<<\n", AlbArt, AblTit, $0
		}
	} else
	{		# album line - check it
		if (length(DEBUGSW)>0) printf "\tAlbum line\n"

		if (Cuts == 0 && NR > 1)
			printf "Line %d - no cuts (%s - %s)\n", NR-1, AlbArt, AlbTit

		Cuts = 0		# no cuts so far
		AlbArt = $ALB_ART
		AlbTit = $ALB_TIT

		if (NF != 5)
		{		# wrong number of fields
			printf "Line %d: Wrong number of fields in album >>%s<<\n", \
						NR, $0
		}
		ChkCap($ALB_ART)
		ChkCap($ALB_TIT)
	}
}

END {
	if (length(AlbArt) > 0 && Cuts == 0)
		printf "Last Album - no cuts (%s - %s)\n", AlbArt, AlbTit
}

func ChkCap(Str) {		# check for strange caps
	len = length(Str)
	if (length(DEBUGSW)>0)
		printf "ChkCap(%s), len=%d\n", Str, len

	Err = 0		# no problems so far

	for (i=1; i < len && Err == 0; i++)
	{		# loop thru cut title - look for strange capitalization
		c1 = substr(Str,i,1)		# get this char
		c2 = substr(Str,i+1,1)		# and the next
        if (c1 == "I" && c2 == "I")
        { # this is two in Roman Numerals - ignore
        }
		else if (index(UPPER,c1) > 0)
		{		# this is upper case - check for strange capitols
			if (index(UPPER,c2) > 0)
			{
				Err = i
				Reas = "Assuming Roman Number"
				for (i=j; j <= length(Str); j++)
				{
					if (index(NOTROMAN,substr(Str,j,1)) > 0)
					{		# not roman number - complain
						Err = i
						Reas = "Two upper case characters"
					}
				}
			} else if (c2 == " ")
			{
				if (FILENAME == "bulgaria.dat" && c1 == "E")
				{
					if (length(DEBUGSW)>0) printf "Word E in line %d\n", NR
				} else if (c1 != "I" && c1 != "A" && c1 != "O" && c1 != "E")
				{
					if (index(NOTROMAN,c1)>0)
					{		# not roman characters, complain
						Err = i
						Reas = "Upper case followed by space"
					} else
					{
						printf "Assuming Roman numbers '%s %s' in %s\n", \
									c1, c2, Str
					}
				}
			}
		} else if (index(LOWER,c1) > 0 && index(UPPER,c2) > 0)
		{		# lower case followed by upper case - Mc or Mac?
			if (i > 1 && substr(Str,i-1,2) == "Mc")
			{
				if (length(DEBUGSW)>0) printf "Mc at position %d\n", i-1
			} else if (i > 2 && substr(Str,i-2,3) == "Mac")
			{
				if (length(DEBUGSW)>0) printf "Mac at position %d\n", i-2
			} else
			{
				Err = i
				Reas = "Lower case followed by upper case"
			}
		}
	}
	if (Err)
	{
		printf "--Line %d: %s in cut %d, char %d [%s]", \
					NR, Reas, Cuts, Err, substr(Str,Err,4)
		printf " (%s - %s) >>%s<<\n", AlbArt, AlbTit, $0
	}
}
END { print "Album " FILENAME " processed" }


