/*
 *				**** BROWSE ****
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 * This file permits the user to browse through the database
 *
 */

#include <stdio.h>
#include "cd.h"

#define	MAXDAT	500
#define	MAXDISP	20
#define	MAXWIDE	81

/* return codes from brws_list */

#define	BRWS_NXT	-1				/* next page */
#define	BRWS_PRV	-2				/* previous page */
#define	BRWS_TOP	-3				/* top page */
#define	BRWS_BOT	-4				/* bottom page */
#define	BRWS_QUT	-5				/* quit loop */

do_browse()
{		/* browse */
	char	*DispName[MAXDISP];		/* list to be displayed */
	int		topfile=0;				/* file at top of screen */
	int		botfile;				/* number of files to display */
	int		opt;					/* option from display list */
	int		ndisp;					/* number of items to display */
	int		i,j;					/* loop variable */
	FILE	*xfp;					/* index file pointer */
	char	*xname;					/* name of index file */

	xname = getcore(mast_name);
	strcpy(xname+strlen(mast_name)-4,".kti");	/* actual file name */
	if ((xfp = fopen(xname,"r")) == NULL)
	{		/* problems */
		perror("browse");
		fprintf(stderr,"Can't open index file %s.\n", xname);
		return(1);
	}

	do
	{		/* keep this up while something to do */
		if ((botfile = topfile + MAXDISP) > tot_dat) botfile = tot_dat;

		for (i=0, j=topfile; i < MAXNDX && j < botfile; i++, j++)
		{		/* build display list */
			DispName[i] = datafile[j].df_fname;
		}

		/*  names copied - call display list */
		ndisp = botfile - topfile;	/* how many to display */
		switch(opt = br_list(DispName,ndisp))
		{		/* process command */
			case BRWS_QUT: return 0;		/* quit */
			case BRWS_TOP:	topfile = 0; break;		/* top of file */
			case BRWS_BOT:
				topfile = tot_dat - MAXDISP;		/* bottom */
				if (topfile < 0) topfile = 0;
				break;

			case BRWS_NXT:
				topfile += MAXDISP;
				if (topfile > botfile) topfile = botfile - MAXDISP;
				if (topfile < 0) topfile = 0;
				break;

			case BRWS_PRV:
				topfile -= MAXDISP;
				if (topfile < 0) topfile = 0;
				break;

			default:
				if (opt >= 1 && opt <= MAXDISP)
					br_dat(topfile + opt -1,xfp);	/* browse this file */
				else
				{		/* problems */
					fprintf(stderr,"Internal error (Browse)! Opt=%d.\n", opt);
				}
		}		/* end of switch */
	} while (opt != BRWS_QUT);
				/* end of browse loop */
	fclose(xfp);
}

br_list(list,num)
char	*list[];			/* list to display */
int		num;				/* number of items in list */
{		/* display list and prompt for command */
	char	buf[BUFSIZ];	/* input buffer */
	int		cmd;			/* commnd from user */
	int		i;				/* loop variable */

	if (num < 1)
	{		/* bad input */
		fprintf(stderr,"Internal error (br_list).  Num=%d.\n", num);
		return BRWS_QUT;
	}

	do
	{
		for (i=0; i < num; i++) printf("%d. %s\n", i+1, list[i]);
		printf("Enter command (#,n,p,t,b,q,?): ");
		gets(buf);			/* get response from user */
		switch(buf[0])
		{		/* process request */
		case '\0':	continue;				/* nothing in, nothing out */
		case 'n':	cmd = BRWS_NXT; break;	/* next page */
		case 'p':	cmd = BRWS_PRV; break;	/* prev page */
		case 't':	cmd = BRWS_TOP; break;	/* top page */
		case 'b':	cmd = BRWS_BOT; break;	/* bottom page */
		case '?':	br_help(num); cmd = 0; continue;	/* help msg */
		case 'q':	cmd = BRWS_QUT; break;	/* quit loop */
		default:	cmd = atoi(buf); break;	/* item number? */
		}

		if (cmd == 0 || cmd > num)
		{		/* invalid option */
			fprintf(stderr,"Invalid option (%s).  Type '?' for help.\n", buf);
			cmd = 0;
			continue;
		}
	} while (cmd == 0);		/* keep looping */

	return cmd;
}

br_dat(num,xfp)
int		num;				/* data file to browse */
FILE	*xfp;				/* index file pointer */
{		/* browse thru data file */
	long	NdxLoc[MAXDAT];	/* location of albums */
	int		opt;			/* option from user */
	int		topfile;		/* top file to display */
	int		botfile;		/* bottom file to display */
	int		lasttop;		/* last top file */
	int		total;			/* total data entries */
	int		ndisp=0;		/* number of display variables */
	int		loc;			/* actual location of entry */
	char	*Disp[MAXDISP];	/* albums to display */

	open_dat(num);			/* open data file */
	total = load_ndx(NdxLoc,num,xfp);	/* load location of albums */
	topfile = 0;			/* start at the top */
	lasttop = -1;			/* force load of struct */

	do
	{		/* loop until quit command */
		if (topfile != lasttop)
		{		/* not the same as last - reset */
			if (lasttop >= 0) free_cuts(Disp,ndisp);
			if ((botfile = topfile + MAXDISP) > total) botfile = total;
			ndisp = botfile - topfile;
			br_cuts(Disp,NdxLoc,datafile[num].df_fp,topfile,botfile);	/* load albums */
			lasttop = topfile;
		}

		switch(opt = br_list(Disp,ndisp))
		{		/* process command request */
		case BRWS_NXT:		/* next page */
			topfile += MAXDISP;
			if (topfile > total) topfile = total - MAXDISP;
			if (topfile < 0) topfile = 0;
			break;

		case BRWS_PRV:		/* previous page */
			topfile -= MAXDISP;
			if (topfile < 0) topfile = 0;
			break;

		case BRWS_TOP:		/* top page */
			topfile = 0;
			break;

		case BRWS_BOT:		/* bottom page */
			topfile = total - MAXDISP;
			if (topfile < 0) topfile = 0;
			break;

		case BRWS_QUT:		/* quit */
			break;

		default:			/* selected an album */
			if (opt >= 1 && opt <= ndisp)
			{
				loc = topfile + opt - 1;
				br_alb(NdxLoc[loc],datafile[num].df_fp);
			} else fprintf(stderr,"Internal error (br_dat). Opt=%d.\n", opt);
		}		/* end switch */
	} while (opt != BRWS_QUT);
}

load_ndx(NdxLoc,num,xfp)
long	NdxLoc[];			/* location table to load */
int		num;				/* data file index to load */
FILE	*xfp;				/* index file */
{		/* load NdxLoc */
	int		i, j=0;			/* loop variables */
	char	buf[BUFSIZ];	/* input buffer */
	char	*flds[KEYFLDS];	/* for splitting fields */
	long	atol();

	rewind(xfp);				/* start at top */

	while (fgets(buf,sizeof(buf),xfp) != NULL)
	{		/* loop thru index file looking for the data file */
		split_flds(buf,flds,'|',0,KEYFLDS);
		if ((i = atoi(flds[1])) < num) continue;	/* not there yet */
		if (i > num) break;				/* past album */
		j = atoi(flds[2]) -1;			/* album number */
		NdxLoc[j] = atol(flds[3]);
	}
	return ++j;			/* number of albums */
}

br_cuts(Disp,NdxLoc,fp,topfile,botfile)
char	*Disp[];			/* list to be loaded */
long	NdxLoc[];			/* location of album lines */
FILE	*fp;				/* input file */
int		topfile;			/* index of first file to load */
int		botfile;			/* index of bottom file to load */
{		/* load struct Disp with entries topfile to botfile in NdxLoc */
	int		i, j;			/* loop variables */
	char	*flds[ARTFLDS];	/* for album fields */
	char	buf[BUFSIZ];	/* input buffer */
	char	buf2[BUFSIZ];	/* display buffer */

	for (i = 0, j = topfile; j < botfile; i++, j++)
	{		/* load entry */
		if (fseek(fp,NdxLoc[j],0))
		{		/* problems */
			perror("br_cuts");
			fprintf(stderr,"Can't seek data file.\n");
			exit(2);
		}
		fgets(buf,sizeof(buf),fp);		/* read line */
		split_flds(buf,flds,'|',0,ARTFLDS);
		sprintf(buf2,"%s -- %s", flds[0],flds[1]);
		buf2[75] = '\0';				/* truncate */
		Disp[i] = getcore(buf2);
	}
}

free_cuts(Disp,ndisp)
char	*Disp[];
int		ndisp;				/* number of entries to free */
{		/* free memory in Disp */
	int		i;				/* loop variable */

	for (i=0; i < ndisp; i++)
	{		/* free memory */
		free(Disp[i]);
	}
}

br_alb(Loc,fp)
long	Loc;		/* location of album */
FILE	*fp;		/* file pointer */
{
	char	buf[BUFSIZ];		/* input buffer */
	int		cut=0;				/* cut number on album */
	char	*p;					/* scan pointer */

	if (fseek(fp,Loc,0))
	{		/* problems */
		fprintf(stderr,"br_alb: Can't seek data file!  Loc=%ld.\n", Loc);
		return;
	}

	if (fgets(buf,sizeof(buf),fp) == NULL)
	{		/* can't read album */
		fprintf(stderr,"br_alb: Can't read album line!\n");
		return;
	}

	fputs(buf,stdout);		/* print album line */

	while (fgets(buf,sizeof(buf),fp) != NULL)
	{		/* still something to read */
		if (!isspace(*buf)) break;		/* next album - done */
		for (p=buf; isspace(*p); p++);	/* find non-blank */
		fprintf(stdout,"%2d. %s\r", ++cut, p);
		if ((cut % 20) == 0)
		{		/* end of screen page */
			printf("Press ENTER to continue: ");
			gets(buf);
		}
	}

	printf("Press ENTER to continue: ");
	gets(buf);
}

br_help(num)
int		num;			/* number of items on screen */
{
	printf("Enter a number from 1 to %d to inspect an item on the screen.\n", num);
	printf("Enter 'n' for next, or 'p' for previous screen.\n");
	printf("Enter 't' for top of list, or 'b' for bottom.\n");
}
