/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** GET_ALBUMS ****
 *
 * This routine obtains a list of albums to keep
 * The user may specify by artist, title or both
 *
 */

#include <stdio.h>
/*#include <io.h>*/
#include <ctype.h>
#include "cd.h"

get_albums()
{
	int		option;
	char	buf[BUFSIZ], *p;

	while (1)
	{		/* loop until quit or exit option */
		printf("\n\nOptions:\n\n");
		printf("L\tList Albums only\n");
		printf("A\tAdd Albums\n");
		printf("D\tDelete Album\n");
		printf("M\tMove Album\n");
		printf("C\tList Albums and Cuts\n");
		printf("\n");
		printf("Z\tClear out all Keep Albums\n");
		printf("T\tGo to Tape Menu\n");
		printf("W\tWrite to a File\n");
		printf("\n");
		printf("?\tHelp Message\n");
		printf("Q\tExit to Main Program\n");
		printf("X\tExit Program\n");

		printf("\nEnter Option: ");
		gets(buf);
		p = clean_up(buf);
		if (*p == EOS) continue;
		option = *p;

		switch(option) {
		case 'X':			/* Exit */
		case 'x':
		case 'Q':			/* Quit */
		case 'q':
			return option;

		case 'L':			/* List Albums */
		case 'l':
			list_albums(stdout,0);
			break;

		case 'A':			/* add albums */
		case 'a':
			add_albums();
			break;

		case 'D':			/* delete albums */
		case 'd':
			del_albums();
			break;

		case 'M':			/* move albums */
		case 'm':
			move_albums();
			break;

		case 'C':			/* print albums and cuts */
		case 'c':
			list_albums(stdout,1);
			break;

		case 'T':			/* go to tape menu */
		case 't':
			option = 'T';
			return option;
			/****** no break needed since this case returns ****/

		case 'W':			/* write out to a file */
		case 'w':
			write_albums();
			break;

		case 'Z':			/* zap keep albums */
		case 'z':
			printf("\nClear out all %d albums being kept? (y or n): ", tkeep);
			gets(buf);
			p = clean_up(buf);
			if (*p == 'y' || *p == 'Y') zap_all();
			break;

		case '?':		/* help */
			help("KEEP");
			break;

		default:		/* bad value */
			printf("Invalid option.  Try again\n");
			option = 0;		/* make sure we don't exit */
			break;
		}	/* end case statement */
	}		/* option loop */
}

list_albums(fp,cuts)
FILE *fp;		/* file to write to */
int	cuts;		/* if we are to list cuts */
{
	int		i;
	char	buf[BUFSIZ];

	fprintf(fp,"%d Albums selected from database.", tkeep);
	if (fp != stdout) fprintf(fp," (Load CD changer in same order)\n");
	else fprintf(fp,"\n");

	for (i=0; i < tkeep; i++)
	{
		list_album(fp,i,cuts);
		if (cuts != 0 && fp == stdout)
		{
			printf("Press ENTER to continue: ");
			gets(buf);
		}
	}
	if (cuts == 0 && fp == stdout)
	{
		printf("Press ENTER to continue: ");
		gets(buf);
	}
}

list_album(fp,i,cuts)
FILE *fp;		/* file pointer */
int	i;			/* position of album to print */
int	cuts;		/* whether to print cuts as well */
{
	fprintf(fp,"\t%3d. %s (by %s)\n",
				i+1, keep[i].ka_name, keep[i].ka_art);
	if (cuts) list_cuts(fp,keep[i].ka_beg,keep[i].ka_end);
}

add_albums()
{
	int	looping;
			/* lists from search by artist & title, plus combined list */
	int		nart_match;				/* how many artists found */
	struct	key_match art_match[MAXMATCH];
	int		ntit_match;				/* how many titles found */
	struct	key_match tit_match[MAXMATCH];
	int		nkey_match;				/* how many combined keys matched */
	struct	key_match key_match[MAXMATCH];

	char	buf[BUFSIZ];		/* input buffer for user input */
	char	*p;					/* parsing pointer */
	char	datbuf[BUFSIZ];		/* input buffer for album file */
	int		i, j;				/* loop variables */
	int		fnum;				/* file number of record */
	long	datloc;				/* location of album record */
	long	cutloc;				/* location of cut record */
	char	*key;				/* hashed key to match */

	int		nkeep;				/* how many albums are being kept */
	int		keep_beg[MAXKEEP];			/* range begin values */
	int		keep_end[MAXKEEP];			/* range end values */
	int		keep_start, keep_stop;		/* actual loop ranges */

	int		nflds;				/* how many fields were found */
	char	*flds[DATFLDS];		/* the fields themselves */

	for (looping=1;looping > 0;)
	{
		looping = 0;		/* reset if either Artist or Title searched */
		printf("Enter Artist (ENTER to skip Artist): ");
		gets(buf);
		p = clean_up(buf);
		if (*p != EOS)
		{
			looping++;		/* try once, try next */
			key = get_key(p);
			nart_match = find_key(art_file,key,art_match,MAXMATCH);
			printf("%d matches for key string >>%s<<\n", nart_match, key);
		} else nart_match = 0;	/* nothing searched, nothing found */

		printf("Enter Title (or ENTER to skip Title): ");
		gets(buf);
		p = clean_up(buf);
		if (*p != EOS)
		{
			looping++;		/* make sure we try again */
			key= get_key(p);
			ntit_match = find_key(tit_file,key,tit_match,MAXMATCH);
			printf("%d matches for key string >>%s<<\n", ntit_match, key);
		} else ntit_match = 0;	/* no lookee, no findee */

			/* set up key_match for combined search result */
		if (looping == 0) continue;		/* didn't even try */
		else if (nart_match == 0 && ntit_match == 0)
		{		/* no matches */
			printf("%s: No matches found\n", Prog);
			continue;
		} else if (nart_match == 0)
		{		/* no Artist, just title */
			nkey_match = ntit_match;		/* number of matches */
			for (i=0; i < nkey_match; i++)
			{
				key_match[i].km_file = tit_match[i].km_file;
				key_match[i].km_rec  = tit_match[i].km_rec;
				key_match[i].km_loc  = tit_match[i].km_loc;
				key_match[i].km_cut	 = tit_match[i].km_cut;
			}
		} else if (ntit_match == 0)
		{		/* no Title, just Artist */
			nkey_match = nart_match;
			for (i=0; i < nkey_match; i++)
			{
				key_match[i].km_file = art_match[i].km_file;
				key_match[i].km_rec  = art_match[i].km_rec;
				key_match[i].km_loc  = art_match[i].km_loc;
				key_match[i].km_cut  = art_match[i].km_cut;
			}
		} else
		{		/* some of both, look for common fields */
			nkey_match = 0;		/* nothing found yet */
			for (i=0; i < nart_match; i++)
			{		/* loop thru all Artist matches */
				for (j=0; j< ntit_match; j++)
				{		/* loop thru all Title matches */
					if (art_match[i].km_file != tit_match[j].km_file) continue;
					if (art_match[i].km_rec != tit_match[j].km_rec) continue;
						/* match found, add to combined */
					key_match[nkey_match].km_file = art_match[i].km_file;
					key_match[nkey_match].km_rec  = art_match[i].km_rec;
					key_match[nkey_match].km_loc  = art_match[i].km_loc;
					key_match[nkey_match].km_cut  = art_match[i].km_cut;
					nkey_match++;
				}		/* end loop thru Title matches */
			}			/* end loop thru Artist matches */
		}

		if (nkey_match == 0)
		{
			printf("No matches found for both keys\n");
			continue;
		}

			/* match found, retreive info */
		for (i=0; i < nkey_match; i++)
		{
			fnum = key_match[i].km_file;		/* file number of entry */
			datloc = key_match[i].km_loc;		/* location of album record */
			cutloc = key_match[i].km_cut;		/* location of cut record */

			if (debugsw) printf("Match %d: File %d, record %ld, loc %ld\n",
				i, fnum, key_match[i].km_rec, key_match[i].km_loc);

			open_dat(fnum);		/* make sure file is open */
			if ((j = fseek(datafile[fnum].df_fp,datloc,SEEK_SET)) != 0)
			{
				fprintf(stderr,"I/O error: could not seek to record %ld in data file %d.\n",
								key_match[i].km_rec, key_match[i].km_file);
				perror(Prog);
				fprintf(stderr,"Seek location %lx, return code %d.\n",
								key_match[i].km_loc, j);
				continue;
			}

				/* positioned to album line - get it */
			if ((p = fgets(datbuf,sizeof(datbuf),datafile[fnum].df_fp)) == NULL)
			{
				fprintf(stderr,"%s: Unable to read record at location %ld in file %d\n",
						Prog, datloc, fnum);
				perror(Prog);
				fprintf(stderr,"Return code was %d.\n", p);
				exit(3);
			}
			p = clean_up(datbuf);
			printf("%4d - %s\n", i+1, p);

			if (datloc != cutloc)
			{		/* not just album - print cut too */
				if ((j = fseek(datafile[fnum].df_fp,cutloc,SEEK_SET)) != 0)
				{
					fprintf(stderr,"I/O error: could not seek to cut for record %ld in data file %d.\n",
									key_match[i].km_rec, key_match[i].km_file);
					perror(Prog);
					fprintf(stderr,"Seek location %lx, return code %d.\n",
									key_match[i].km_loc, j);
					continue;
				}

					/* positioned to cut line - get it */
				if ((p = fgets(datbuf,sizeof(datbuf),datafile[fnum].df_fp)) == NULL)
				{
					fprintf(stderr,"%s: Unable to read cut for record at location %ld in file %d\n",
							Prog, datloc, fnum);
					perror(Prog);
					fprintf(stderr,"Return code was %d.\n", p);
					exit(3);
				}
				p = clean_up(datbuf);
				printf("\t%s\n", p);		/* print cut line */
			}

		}		/* end loop thru matches */

		printf("Enter number of album(s) to keep: ");
		gets(buf);
		p = clean_up(buf);
		if (*p == EOS) continue;
			/* get list of numbers */
		if ((nkeep = parse_nums(p,keep_beg,keep_end,MAXKEEP)) == 0) continue;

		for (i=0; i < nkeep; i++)
		{		/* add these albums to list */

			keep_start = keep_beg[i] - 1;
			keep_stop  = keep_end[i] - 1;
			if (keep_start < 0 || keep_start >= nkey_match
				|| keep_start > keep_stop || keep_stop >= nkey_match)
			{
				printf("Range %d is invalid\n", i+1);
				return;
			}

			for (j=keep_start; j <= keep_stop; j++)
			{
				if (tkeep >= MAXKEEP)
				{
					fprintf(stderr,"%s: Only %d albums may be kept\n",
							Prog, MAXKEEP);
					break;
				}

				printf("Keeping album %d in position %d\n", j+1, tkeep+1);
				fnum = key_match[j].km_file;
				datloc = key_match[j].km_loc;

					/* position to album */
				open_dat(fnum);		/* make sure file is open */
				fseek(datafile[fnum].df_fp,datloc,SEEK_SET);
				if (fgets(datbuf,sizeof(datbuf),datafile[fnum].df_fp) == NULL)
				{
					fprintf(stderr,"Internal error. Couldn't seek.\n");
					perror(Prog);
					exit(4);
				}

				p = clean_up(datbuf);
				if (*p == EOS)
				{
					fprintf(stderr,"Internal error.  Album line is empty!\n");
					exit(4);
				}

				if ((nflds = split_flds(datbuf,flds,'|',0,DATFLDS)) < 3)
				{
					fprintf(stderr,"Internal error.  Album has only %d fields.\n", nflds);
					exit(4);
				}

					/* load album and artist names */
				if (*flds[0] != EOS)
					keep[tkeep].ka_art = getcore(flds[0]);		/* save artist */
				else keep[tkeep].ka_art = NULL;

				if (*flds[1] != EOS)
					keep[tkeep].ka_name = getcore(flds[1]);	/* save name */
				else keep[tkeep].ka_name = NULL;

				keep[tkeep].ka_beg = tcuts;		/* next free loc in cut table */
				keep[tkeep].ka_end = load_cuts(datafile[fnum].df_fp,tkeep);	/* ends here */
				list_album(stdout,tkeep,1);			/* show what we loaded */
				tkeep++;
			}
		}
	}			/* user entered null string */
}

load_cuts(fp,nrec)
FILE	*fp;		/* input file */
int		nrec;		/* what album number we are adding */
{				/* read cuts into cuts structure table */
	char	buf[BUFSIZ];		/* input buffer */
	char	*flds[MAXCUTS];		/* fields for cut record */
	int		nflds;				/* how many cut fields found */
	int		ncut;				/* what cut number we are on */
	char	*p;					/* temp ptr for parsing */
	int		tmp;				/* temp var or anything */

	ncut = 0;
	while (1)
	{		/* loop thru file */
		if (fgets(buf,sizeof(buf),fp) == NULL) break;	/* EOF */

		p = clean_up(buf);		/* get rid of CR-LF */
		if (*buf == EOS) continue;		/* empty line, ignore it */
		if (!isspace(*buf)) break;		/* no longer a cut */
		if (tcuts >= MAXCUTS)
		{
			fprintf(stderr,"Can only store %d cuts\n", MAXCUTS);
			break;
		} else ncut++;				/* one more cut */

			/* we now have the cut line in buf - parse & check it */
			/* p points to the first non-whitespace character */
		nflds = split_flds(p,flds,'|',0,CUTFLDS);
		if (nflds < 1 || nflds > CUTFLDS)
		{
			fprintf(stderr,"Line for Cut %d has %d fields\n", ncut, nflds);
			if (nflds < 1) continue;
		}

			/* we now have a good cut line, load it */
		cuts[tcuts].cut_nrec = nrec;		/* what album it came from */
		cuts[tcuts].cut_num  = ncut - 1;		/* what cut it is */

		cuts[tcuts].cut_name = getcore(flds[0]);

		tmp = 0;			/* assume zero time */
		if (nflds == 1 || strlen(flds[1]) == 0)
		{
			fprintf(stderr,"No time given for cut %d\n", ncut);
		} else
		{
			if ((tmp = parse_time(flds[1])) < 0)
			{
				fprintf(stderr,"Invalid time (%s) for cut %d ignored\n",
					flds[1], ncut);
			}
			cuts[tcuts].cut_min = tmp / 60;		/* integer divide */
			cuts[tcuts].cut_sec = tmp % 60;		/* modulo for seconds */
		}		/* time field processed */

		if (nflds > 2 && strlen(flds[2]) > 0)
			cuts[tcuts].cut_art = getcore(flds[2]);
		else cuts[tcuts].cut_art = getcore(keep[nrec].ka_art);
		tcuts++;			/* increment cut counter */
	}   	/* end loop thru file */
	return tcuts;
}

del_albums()
{
	int		i, j;
	int		delcnt, delbeg[MAXKEEP], delend[MAXKEEP];
	int		delpos, deltot;
	char	buf[BUFSIZ];

	if (tkeep < 1)
	{
		printf("Nothing to delete\n");
		return;
	}

	printf("Enter positions (between 1 and %d) to delete: ", tkeep);
	gets(buf);
	if ((delcnt = parse_nums(buf,delbeg,delend,MAXKEEP)) < 1) return;

	for (i=0; i < delcnt; i++)
	{		/* loop thru ranges given */
		for (delpos = delbeg[i]; delpos <= delend[i]; delpos++)
		{		/* loop thru albums in this range */
			if (delpos < 1 || delpos > tkeep)
			{
				printf("Invalid range entered (%d to %d).\n",
					delbeg[i], delend[i]);
					return;
			}
			list_album(stdout,delpos-1,1);
		}		/* end loop thru albums in this range */
	}			/* end loop thru ranges */

	j = 0;		/* how many sides have stuff on them */
	for (i=0; i < MAXSIDES; i++)
	{
		if (tapes[i].ts_num > 0)
		{
			tape_used(stdout,i);
			j++;
		}
	}
	if (j > 0)
		printf("\n*** Warning!! Deleting albums will clear out tapes.\n");

	printf("Delete? (y or n): ");
	gets(buf);
	if (*buf != 'y' && *buf != 'Y')
	{
		printf("Delete cancelled.\n");
		return;
	}

		/* OK, we want to delete these, go thru them again */

	for (i=0; i < delcnt; i++)
	{		/* loop thru ranges */
		for (delpos=delbeg[i]; delpos <= delend[i]; delpos++)
		{		/* delete album - first delete cuts */
			for (j=keep[delpos-1].ka_beg; j < keep[delpos-1].ka_end; j++)
			{		/* free up space for cuts */
				free_up(cuts[j].cut_name); cuts[j].cut_name = NULL;
				free_up(cuts[j].cut_art);  cuts[j].cut_art  = NULL;
			}
				/* cuts deleted, delete album */
			free_up(keep[delpos-1].ka_name); keep[delpos-1].ka_name = NULL;
			free_up(keep[delpos-1].ka_art);  keep[delpos-1].ka_art  = NULL;
			printf("Album %d deleted\n", delpos);
		}	/* end loop thru albums in this range */
	}		/* end loop thru ranges */

			/* now move rest of entries up to close up albums */
			/* notice that the cuts are not adjusted */
			/* If necessary, we can add a compress cuts command */

		deltot = 0;		/* no albums yet */
		for (i=0; i < tkeep; i++)
		{		/* loop thru entire album space */
			if (keep[i].ka_name == NULL) continue;	/* was deleted */
				/* we have a good album here, copy it into next free slot */
			if (deltot == i)
			{	/* already there */
				deltot++;		/* update album count */
				continue;
			}

				/* copy good entry to next slot */
			keep[deltot].ka_name=  keep[i].ka_name;
			keep[deltot].ka_art  = keep[i].ka_art;
			keep[deltot].ka_beg  = keep[i].ka_beg;
			keep[deltot].ka_end  = keep[i].ka_end;

				/* clear out old position */
			keep[i].ka_name=  NULL;
			keep[i].ka_art  = NULL;
			keep[i].ka_beg  = -1;
			keep[i].ka_end  = -1;

			deltot++;			/* update counter of albums left */
		}
		tkeep = deltot;			/* albums left */
		zap_tape(0);			/* zap tape sides */
}


move_albums()
{
	int		oldloc, newloc;
	int		i;
	char	buf[BUFSIZ];
	struct	keep	key_save;

	if (tkeep < 1) return;		/* nothing there */
	printf("Enter position of album to move (or zero to cancel): ");
	gets(buf);
	oldloc = atoi(buf);
	if (oldloc == 0) return;
	else oldloc--;
	if (oldloc >= tkeep)
	{
		printf("There are only %d albums\n", tkeep+1);
		return;
	}
	list_album(stdout,oldloc,0);

	printf("Enter where to move album #%d (or zero to cancel): ",oldloc);
	gets(buf);
	newloc = atoi(buf);
	if (newloc == 0) return;
	else if (newloc > tkeep)
	{
		printf("There are only %d albums\n", tkeep+1);
		return;
	} else newloc--;

	if (oldloc == newloc) return;

		/* save what we are to move */
	key_save.ka_name = keep[oldloc].ka_name;
	key_save.ka_art = keep[oldloc].ka_art;
	key_save.ka_beg  = keep[oldloc].ka_beg;
	key_save.ka_end  = keep[oldloc].ka_end;

		/* now shift things around to make room for the new entry */
	if (oldloc < newloc)
	{
		for (i=oldloc; i <= newloc; i++)
		{
			keep[i].ka_name  = keep[i+1].ka_name;
			keep[i].ka_art = keep[i+1].ka_art;
			keep[i].ka_beg  = keep[i+1].ka_beg;
			keep[i].ka_end  = keep[i+1].ka_end;
		}
	} else
	{
		for (i=oldloc-1; i >=newloc; i--)
		{
			keep[i+1].ka_name  = keep[i].ka_name;
			keep[i+1].ka_art = keep[i].ka_art;
			keep[i+1].ka_beg  = keep[i].ka_beg;
			keep[i+1].ka_end  = keep[i].ka_end;
		}
	}

	keep[newloc].ka_name  = key_save.ka_name;
	keep[newloc].ka_art = keep[newloc].ka_art;
	keep[newloc].ka_beg  = key_save.ka_beg;
	keep[newloc].ka_end  = key_save.ka_end;

	printf("Album %d moved to position %d\n", oldloc+1, newloc+1);
}

write_albums()
{
	FILE	*fp;
	char	fname[BUFSIZ], buf[BUFSIZ], *p;
	char	*open_mode;
	int		i, j, k;

	printf("Enter name of file (or ENTER to cancel): ");
	gets(fname);
	p = clean_up(fname);
	if (*p == EOS) return;
	else if (p > fname) strcpy(fname,p);	/* shift whitespace out */

	if (access(fname,0) == 0)
	{
		printf("File %s exists.  Append? (y or n): ");
		gets(buf);
		p = clean_up(buf);
		if (*p == 'n' || *p == 'N') open_mode = FMODE_WRITE;
		else open_mode = FMODE_APP;
	} else open_mode = FMODE_WRITE;

	if ((fp = fopen(fname,open_mode)) != NULL)
	{		/* open succeeded, write configuration */
		for (i=0; i < tkeep; i++)
		{
			fprintf(fp,"%3d - %s - %s\n", i+1, keep[i].ka_name, keep[i].ka_art);
			j = 0;				/* no cuts yet */

			for (k=keep[i].ka_beg; k < keep[i].ka_end; k++)
			{
				fprintf(fp,"\t%2d. %s|%.1d:%2.2d",
					++j, cuts[k].cut_name, cuts[k].cut_min, cuts[k].cut_sec);
				if (cuts[k].cut_art != NULL) fprintf(fp,"|%s\n", cuts[k].cut_art);
				else fprintf(fp,"\n");
			}		/* end of loop thru cuts */
		}			/* end of loop thru albums */
		fclose(fp);	/* clean up file */
	} else
	{		/* open failed */
		fprintf(stderr,"Unable to open output file %s\n", fname);
		perror(Prog);
	}
}

list_cuts(fp,beg,end)
FILE *fp;		/* file pointer */
int	beg;		/* starting location in cut table */
int	end;		/* ending location in cut table */
{
	int	i;		/* good old i */
	int	ttime;	/* total time for album */

	if (beg < 0 || beg > tcuts || beg > end || end > tcuts) return;
	ttime = 0;

	for (i=beg; i < end; i++)
	{		/* loop thru cut table */
		fprintf(fp,"\t%2d. %2d:%2.2d %s",
			i-beg+1, cuts[i].cut_min, cuts[i].cut_sec, cuts[i].cut_name);
		if (cuts[i].cut_art == NULL) fprintf(fp,"\n");
		else fprintf(fp," (%s)\n", cuts[i].cut_art);
		ttime += (cuts[i].cut_min*60) + cuts[i].cut_sec;
	}
	fprintf(fp,"\t    %s Total Time\n", str_time(ttime));
}

zap_all()
{		/* clear out albums */
	int	i, j;

	for (i=0; i < tkeep; i++)
	{		/* clear each album */
		free_up(keep[i].ka_name);
		free_up(keep[i].ka_art);
		for (j=keep[i].ka_beg; j < keep[i].ka_end; j++)
		{
			free_up(cuts[j].cut_name);
			free_up(cuts[j].cut_art);
			cuts[j].cut_min = 0;
			cuts[j].cut_sec = 0;
		}
		keep[i].ka_beg = 0;
		keep[i].ka_end = 0;
	}
	tkeep = 0;
}
