/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** OPEN MAST ****
 *
 * This routine opens the master file, and then opens the data files
 */

#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include "cd.h"

open_mast(mast_name)
char	*mast_name;		/* master file name */
{
	int		i;			/* loop variable */
	char	*p;			/* temp ptr */
	FILE	*mast_file;
	char	art_name[BUFSIZ], tit_name[BUFSIZ], cut_name[BUFSIZ];
	char	dirname[BUFSIZ];
	char	buf[BUFSIZ];
	char	datbuf[BUFSIZ];

	long	key_time, dat_time;		/* time of key and data files */
	struct	stat	stbuf;			/* for access times */

	int		nflds;
	char	*flds[KEYFLDS];

	long	tot_art, tot_tit;		/* total albums from artist & title key files */

	if ((p = strrchr(mast_name,'.')) == NULL)
	{
		fprintf(stderr,"%s: Master file must end in .mst\n", Prog);
		return 0;
	}

	if ((mast_file = fopen(mast_name,"r")) == NULL)
	{
		fprintf(stderr,"Can't open master file %s\n", mast_name);
		perror(Prog);
		return 0;
	}

	strcpy(dirname,mast_name);
	for (p=dirname+strlen(dirname)-1; p > dirname && *p != '/' && *p != '\\'; p--);
	if (p > dirname) p++;		/* point after slash */
	*p = EOS;			/* only directory name or null */

	tot_dat = 0;				/* no files found yet */
	dat_time = 0L;				/* latest modification time of data files */
	while (fgets(buf,sizeof(buf),mast_file) != NULL)
	{		/* read names of data files */
		clean_up(buf);		/* clean up buffer */
		if (*buf == EOS) continue;
		if (tot_dat >= MAXNDX)
		{
			fprintf(stderr,"%s: More than %d index files in %s\n",
					Prog, MAXNDX, mast_name);
			exit(1);
		}

		if ((nflds = split_flds(buf,flds,'|',0,MASTFLDS)) != MASTFLDS)
		{		/* wrong number of fields */
			fprintf(stderr,"%s: %d fields in master record %d instead of %d\n",
					Prog, nflds, tot_dat, MASTFLDS);
			continue;
		}

		strcpy(datbuf,dirname);				/* copy in directory name */
		strcat(datbuf,flds[0]);				/* get data file name */
		datafile[tot_dat].df_fname = getcore(datbuf);	/* save name of data file */

		if (stat(datafile[tot_dat].df_fname,&stbuf) != 0)
		{
			fprintf(stderr,"Data file %s not found.\n", datafile[tot_dat].df_fname);
			exit(1);
		} else if (stbuf.st_mtime > dat_time) dat_time = stbuf.st_mtime;
		tot_dat++;			/* number of files so far */
	}

			/* master read, get key names */
	strcpy(art_name,mast_name);
	p = strrchr(art_name,'.');		/* find the suffix again */
	strcpy(p,".sar");		/* artist key file */
	strcpy(tit_name,mast_name);
	p = strrchr(tit_name,'.');
	strcpy(p,".sti");		/* title key file */
	strcpy(cut_name,mast_name);
	p = strrchr(cut_name,'.');		/* find the suffix again */
	strcpy(p,".scu");		/* artist key file */

	if (stat(art_name,&stbuf) != 0)
	{
		fprintf(stderr,"Artist file (%s) not found.\n", art_name);
		exit(1);
	}
	key_time = stbuf.st_mtime;

	if (stat(tit_name,&stbuf) != 0)
	{
		fprintf(stderr,"Title file (%s) not found.\n", tit_name);
		exit(1);
	}


	if (stat(cut_name,&stbuf) != 0)
	{
		fprintf(stderr,"Cut file (%s) not found.\n", cut_name);
		exit(1);
	}
	if (stbuf.st_mtime > key_time) key_time = stbuf.st_mtime;

	if (dat_time > key_time) rebuild_keys(mast_name);

	if ((art_file = fopen(art_name,"r")) == NULL)
	{
		fprintf(stderr,"Unable to open Artist key file %s\n",
					art_name);
		perror(Prog);
		exit(1);
	}

	fgets(buf,sizeof(buf),art_file);		/* read first record */
	if ((nflds = split_flds(buf,flds,'|',0,KEYFLDS)) != KEYFLDS)
	{
		fprintf(stderr,"Artist key file contains %d fields instead of %d\n",
			 nflds, KEYFLDS);
		exit(3);
	}
	tot_art = atol(flds[KEY_LOC]);		/* total albums */
	printf("%ld Albums in database\n", tot_art);

	if ((tit_file = fopen(tit_name,"r")) == NULL)
	{
		fprintf(stderr,"Unable to open Title key file %s\n",
					tit_name);
		perror(Prog);
		exit(1);
	}

	fgets(buf,sizeof(buf),tit_file);		/* read first record */
	if ((nflds = split_flds(buf,flds,'|',0,KEYFLDS)) != KEYFLDS)
	{
		fprintf(stderr,"Title key file contains %d fields instead of %d\n",
			 nflds, KEYFLDS);
		exit(3);
	}


	tot_tit = atol(flds[KEY_LOC]);		/* total albums */
	if (tot_art != tot_tit)
	{
		fprintf(stderr,"Title key file claims %ld albums instead of %ld\n",
			tot_tit, tot_art);
		exit(3);
	}

	tot_albums = tot_art;	/* number of albums */

	if ((cut_file = fopen(cut_name,"r")) == NULL)
	{
		fprintf(stderr,"Unable to open Cut key file %s\n",
					cut_name);
		perror(Prog);
		exit(1);
	}

	fgets(buf,sizeof(buf),cut_file);		/* read first record */
	if ((nflds = split_flds(buf,flds,'|',0,KEYFLDS)) != KEYFLDS)
	{
		fprintf(stderr,"Title key file contains %d fields instead of %d\n",
			 nflds, KEYFLDS);
		exit(3);
	}

	for (i=0; i < max_open && i < tot_dat; i++)
	{		/* loop thru data files & open them */
		if ((datafile[i].df_fp = fopen(datafile[i].df_fname,"r")) == NULL)
		{		/* reached max files open */
			fprintf(stderr,"Unable to open data file %s.\n",
						datafile[i].df_fname);
			exit(2);
		} else datafile[i].df_time = time(NULL);
	}		/* end loop thru data files in master file */

	printf("%d Data Files in database\n", tot_dat);

	/* if max_open still zero, then all files were opened */
	if (max_open > tot_dat) max_open = tot_dat;	/*  all can be open */
	else printf("%d data files open at one time.\n", max_open);
}

rebuild_keys(mname)
char	*mname;
{
	fprintf(stderr,"Index files out of date for mast file %s!\n",mname);
	fprintf(stderr,"Run bld_key to rebuild keys.\n");
	exit(1);
}

open_dat(fnum)
int		fnum;			/* data file number */
{		/* open file if needed and set access time */
	if (datafile[fnum].df_fp == NULL) 
	{		/* not open - retire the least active data file */
		age_dat();
		if ((datafile[fnum].df_fp = fopen(datafile[fnum].df_fname,"r")) == NULL)
		{		/* real problems - can't open data file */
			perror("open_dat");
			fprintf(stderr,"Can't open data file %s.\n", datafile[fnum].df_fname);
			exit(2);
		}
	}

	/* reset access time on file */
	datafile[fnum].df_time = time(NULL);
}

close_dat(fnum)
int		fnum;				/* data file number */
{		/* close data file */
		/* close file if not already closed */
	if (datafile[fnum].df_fp != NULL) fclose(datafile[fnum].df_fp);
	datafile[fnum].df_fp = NULL;		/* signal closed */
	datafile[fnum].df_time = 0L;		/* zap access time */
}

age_dat()
{		/* close data file with oldest access time */
	long	oldtime=0L;			/* oldest time */
	int		oldloc=-1;			/* location of oldest file */
	int		i;					/* loop variable */

	for (i=0; i < tot_dat; i++)
	{		/* loop thru all files */
		if (datafile[i].df_fp == NULL) continue;	/* skip closed */
		if (oldloc < 0)
		{		/* first time - save location */
			oldloc = i;
			oldtime = datafile[i].df_time;
		} else if (oldtime > datafile[i].df_time)
		{		/* new older file */
			oldloc = i;
			oldtime = datafile[i].df_time;
		}
	}

	/* oldest file in oldloc - close it */
	if (oldloc < 0)
	{		/* problems - nothing to close */
		fprintf(stderr,"age_dat: No data files to close!\n");
		exit(2);
	}

	close_dat(oldloc);		/* close oldest file */
}
