/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** PARSE TIME ****
 *
 * This routine will convert a character string representing time
 *	HH:MM into minutes (or MM:SS into seconds)
 *
 * Return codes:
 *	0	Empty string
 *	-1	bad hours field
 *	-2	bad separator (not ':' or '.')
 *	-3	bad minutes field
 *
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>

#ifndef EOS
#define	EOS	'\0'
#endif

parse_time(str)
char	*str;
{
	char	*p, *q;
	int		hr, min;

	for (p=str; *p != EOS && isspace(*p); p++);	/* find first non-blank */
	if (*p == EOS) return 0;		/* empty string */
	else if (!isdigit(*p)) return -1;	/* bad hours */

	for (q=p; isdigit(*q); q++);	/* find first non-numeric */
	if (*q != ':' && *q != '.') return -2;	/* invalid separator */
	if (*q == '.') *q = ':';		/* force a colon for atoi */
	hr = atoi(p);					/* get the hours */

	for (p=q+1; isspace(*p); p++);		/* find first non-blank */
	if (! isdigit(*p)) return -3;		/* bad minutes */
	for (q=p; isdigit(*q); q++);		/* find end of minutes */
	min = atoi(p);						/* get the minutes */
	for (p=q; isspace(*p); p++);		/* check for trailing garbage */
	if (*p != EOS) return -3;			/* garbage found */
	return hr * 60 + min;
}

char *
str_time(sec)
int	sec;
{		/* perform the inverse of parse_time */
	int		min;
	static char	ret[9];

	if (sec < 0) sec = -sec;
	min = sec / 60;
	sec %= 60;

	sprintf(ret,"%2.2d:%2.2d", min, sec);
	return ret;
}
