#!/usr/local/pub/bin/gawk -f
# this script displays the title & name in 80 chars max.

BEGIN {
	FS = "|"

	# album line

	ARTIST		= 1		# name of artist
	TITLE		= 2		# title of album
	DATE		= 3		# date published
	LABEL		= 4		# label name
	CATALOG		= 5		# catalogue number

	# cut line
	CUTNAME		= 1		# name of cut
	TIME		= 2		# time of cut
	CUTART		= 3		# artist, if different than album line

	# max fields
	MAXLEN		= 76	# max size (excluding delimeters)
	MAXART		= 30	# max. artist size
	MAXTIT		= 30	# max. title size
	MAXLBL		= 8		# max. label name size
	MAXCAT		= 8		# max. catalog # size
}

substr($0,1,1) == "\t" { next }		# skip cut lines

{		# album line
	if (FILENAME != LASTFILE)
	{		# new file - display file name
		Name = FILENAME
		if (Name == "popa.alb") Name = "pop"
		if (substr(Name,1,3) != "pop" || substr(Name,5) != ".alb")
			printf "**************** %s ****************\n", Name
		LASTFILE = FILENAME
	}

	Artist = $ARTIST
	Title = $TITLE
	Label = $LABEL
	Catalog = $CATALOG
	alen = length(Artist)
	tlen = length(Title)
	llen = length(Label)
	clen = length(Catalog)
	if (alen == 0 || tlen == 0) next
	reclen = alen + tlen + llen + clen	# size of record

	# if all fields within range, then use max for all fields
	if (alen < MAXART && tlen < MAXTIT && llen < MAXLBL && clen < MAXCAT)
	{		# all fields within range - use max
		alen = MAXART
		tlen = MAXTIT
		llen = MAXLBL
		clen = MAXCAT
	} else
	{		# field(s) too long - shorten something
		if (reclen < MAXLEN)
		{ # some fields too long, but record will fit
			tot = 0	# see how much we are over
			if (alen > MAXART) tot += alen - MAXART
			if (tlen > MAXTIT) tot += tlen - MAXTIT
			if (llen > MAXLBL) tot += llen - MAXLBL
			if (clen > MAXCAT) tot += clen - MAXCAT

			# tot is how much we are over

			if (tot == 0 && tlen < MAXTIT) tlen = MAXTIT
			else if (tlen < MAXTIT && tot > 0)
			{ # title shorter - reduce its size
				diff = MAXTIT - tlen	# how short is title
				if (diff > tot)
				{ # title has enough slack for rest
					tlen = MAXTIT - tot
					tot = 0
				} else
				{ # take up some more slack with the title
					tot -= diff
				}
			}

			if (tot == 0 && alen < MAXART) alen = MAXART
			else if (alen < MAXART && tot > 0)
			{ # artist shorter - reduce its size
				diff = MAXART - alen	# how short
				if (diff > tot)
				{ # enough slack in just artist
					alen = MAXART - tot
					tot = 0
				} else
				{
					tot -= diff	# reduce by difference
				}
			}

			if (tot == 0 && llen < MAXLBL) llen = MAXLBL
			else if (llen < MAXLBL && tot > 0)
			{ # label shorter - reduce its size
				diff = MAXLBL - llen	# how short is label name
				if (diff > tot)
				{ # make up rest of difference with label name
					llen = MAXLBL - tot
					tot = 0
				} else
				{ # take up some more slack with label name
					tot -= diff
				}
			}

			if (tot == 0 && clen < MAXCAT) clen = MAXCAT
			else if (clen < MAXCAT && tot > 0)
			{ # catalog number shorter - make up difference
				diff = MAXCAT - clen
				if (diff > tot)
				{ # make up rest of difference with catalog number
					clen = MAXCAT - tot
					tot = 0
				} else
				{ # this should never happen
					printf "****ERROR**** couldn't fit fields into record\n"
					printf "\talen=%d, tlen=%d, llen=%d, clen=%d\n",
							alen, tlen, llen, clen
					printf "Record >>%s<<\n", $0
				}
			}
		} else
		{ # something has to be shortened
			diff = reclen - MAXLEN	# how much to cut
			# see if we have any slack in any fields
				
			# if artist within range, just shorten title
			if (alen <= MAXART)
			{	# only title too long - shorten it
				Title = substr(Title,1,tlen - diff)	# don't cut artist
				tlen = length(Title)
			} else if (tlen <= MAXTIT)
			{	# only title too long - shorten it
				Artist = substr(Artist,1,alen - diff)
				alen = length(Artist)
			} else
			{		# both too long - truncate both
				Artist = substr(Artist,1,MAXART)
				Title = substr(Title,1,MAXTIT)
				alen = MAXART
				tlen = MAXTIT
			}
		}
	}
	# build format statement
	Fmt = "%-" alen "s|%-" tlen "s|%-" llen "s|%-" clen "s\n"
	printf Fmt, Artist, Title, Label, Catalog
}

func Time(str) {		# compute time in seconds
	if ((x = split(str,Flds,":")) != 2) return 0
	else return (Flds[1] * 60 + Flds[2])
}

func PrtAlb() {	# print last album
	print Album "|" OutTime(TotTime)
}

func OutTime(str) {		# convert seconds to Min:Sec
	Sec = str % 60
	Min = (str - Sec) / 60

	return (Min ":" Sec)
}

func CleanUp(str) {	# clean CR from line if there
	if (substr(str,length(str)) == "\r") ret = substr(str,1,length(str)-1)
	else ret = str
	return ret
}
