/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** TEST INDEX ****
 *
 * This program tests the composition of the key and index files
 *	for the CD program by Bill Pringle.
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <string.h>
#include "cd.h"

/*** global variables within this file ***/
static	int		verbose;				/* verbose flag */
static	char	keyfld[BUFSIZ];				/* key field */
static	int		fnum;						/* file number */
static	int		album;						/* album number */
static	long	ndxloc;						/* location in index file */
static	long	datloc;					/* location in data file */
static	int		nflds;						/* number of fields */
static	char	*flds[KEYFLDS];				/* for key fields */
static	FILE	*efp;						/* error file pointer */
static	FILE	*ep;						/* error message pointer */
static	char	mast_name[BUFSIZ];		/* master file name */
static	char	datbuf[BUFSIZ];				/* data buffer */

main(argc,argv)
int	argc;
char	**argv;
{
	char	buf[BUFSIZ];			/* input buffer */
	char	*p;						/* handy pointer */
	int		i;						/* good old i */
	char	*fmode;					/* file mode */

	int		option;					/* test option */

		/* get my name in lower case w/o full path name */
	p = argv[0] + strlen(argv[0]) - 1;		/* last char */
	while (p >argv[0] && *p != '/' && *p != '\\') p--;
	Prog = (p == argv[0]) ? p : ++p;
	while (*p != EOS)
	{
		if (isupper(*p)) *p = tolower(*p);
		p++;
	}
	argc--;			/* shift program name out */
	argv++;

	if (argc > 0)
	{
		p = clean_up(argv[0]);
		if (*p != EOS) new_mast(p);
		argc--;
		argv++;
	}

	verbose = 0;		/* start off verbose */
	*mast_name = EOS;	/* first time, clear master file name */

	ep = stderr;		/* file pointer for error messages */
	efp = NULL;			/* no error file yet */

	while (1)
	{			/* loop until quit option */
		printf("\nOptions:\n");
		printf("M\tOpen Master file\n");
		printf("K\tRead Key FIle & display album records.\n");
		printf("E\tSet Error File\n");
		printf("V\tSet Verbose Level\n");
		printf("\nQ\tQuit\n");
		printf("\nEnter option: ");
		gets(buf);
		p = clean_up(buf);
		option = *p;
		switch(option) {

		case 'm':
		case 'M':
			printf("Enter master name: ");
			gets(buf);
			p = clean_up(buf);
			if (*p == EOS) continue;
			else new_mast(p);
			break;

		case 'k':
		case 'K':
			test_key();
			break;

		case 'q':
		case 'Q':
			exit(0);

		case 'e':
		case 'E':
			printf("\nEnter error file name (or * for stderr): ");
			gets(buf);
			p = clean_up(buf);
			if (strcmp(p,"*") == 0) ep = stderr;
			else
			{
				if (efp == NULL)
				{
					fmode = FMODE_WRITE;			/* clobber it */
					if ((efp = fopen(p,fmode)) == NULL)
					{
						fprintf(stderr,"Unable to open error file %s.\n", p);
						perror(Prog);
						prt_err(1);
						continue;
					}
				}
				ep = efp;		/* output to a file instead */
			}
			break;

		case 'v':
		case 'V':
			printf("\nVerbose is currently %d, enter new value: ", verbose);
			gets(buf);
			if (*(p = clean_up(buf)) == EOS) continue;
			verbose = atoi(p);
			break;

		default:
			printf("\nInvalid option.  Try again.\n");
		}
	}
}

test_key()
{
	int		rec;						/* record number in index file */
	char	buf[BUFSIZ];				/* input buffer */
	char	*p;							/* pointer var */
	int		ret;						/* return code value */
	char	*retp;						/* return code for pointer vals */
	char	basename[BUFSIZ];			/* base name */
	char	fname[BUFSIZ];				/* key file name */
	char	msg[BUFSIZ];				/* message buffer */
	FILE	*fp;						/* index file pointer */
	long	keyloc;						/* data location from key file */
	long	atol();						/* convert ascii to long */

	printf("\nSearch Artist or Title file (a or t): ");
	gets(fname);
	p = clean_up(fname);
	if (*p == EOS) return;

	if (*p == 'a' || *p == 'A') fp = art_file;
	else if (*p == 't' || *p == 'T') fp = tit_file;
	else
	{
		printf("\nInvalid file type.\n");
		return;
	}

	for (rec = 0; (fgets(buf,sizeof(buf),fp) != NULL); rec++)
	{		/* loop thru all the records in the index file */
		nflds = split_flds(buf,flds,'|',0,KEYFLDS);
		if (nflds != KEYFLDS)
		{
			sprintf(msg,"Record %d has %d fields instead of %d.\n",
							rec, nflds, KEYFLDS);
			prt_msg(1,msg);
		}

			/* get field values */
		strcpy(keyfld,flds[0]);
		fnum	= atoi(flds[1]);
		album	= atoi(flds[2]);
		datloc	= atol(flds[3]);
		sprintf(msg,"%3d. Key %s<<, File %d, Album %d\n",
						rec, keyfld, fnum, album);
		prt_msg(0,msg);

		if ((ret = fseek(datfile[fnum],datloc,SEEK_SET)) != 0)
		{
			fprintf(stderr,"Can't seek to record %d in data file.\n", rec);
			perror(Prog);
			if (prt_err(1)) continue;
		}

		if ((retp = fgets(datbuf,sizeof(buf),datfile[fnum])) == NULL)
		{
			fprintf(stderr,"Can't read record %d in data file.\n", rec);
			perror(Prog);
			if (prt_err(1)) continue;
		 }

		prt_msg(0,datbuf);
	}
}

prt_msg(lev, msg)
int		lev;		/* message level */
char	*msg;		/* message text */
{
	if (lev >= verbose) fputs(msg,ep);
}

new_mast(new_mast)
char	*new_mast;						/* new master file */
{
	char	buf[BUFSIZ];
	char	*p;
	int		i;

	if (*mast_name != EOS)
	{		/* not first time, close files */
		fclose(art_file);
		fclose(tit_file);
	}

	p = clean_up(new_mast);
	strcpy(mast_name,p);		/* save mast name */
		strcpy(buf,mast_name);		/* get file name */
	p = strrchr(buf,'.');		/* find beginning of suffix */
	strcpy(p,".sar");			/* artist key file */
	if ((art_file = fopen(buf,"r")) == NULL)
	{
		fprintf(stderr,"Can't open artist file %s.\n", buf);
		perror(Prog);
		prt_err(1);
		exit(1);
	}

	strcpy(p,".sti");			/* title key file */
	if ((tit_file = fopen(buf,"r")) == NULL)
	{
		fprintf(stderr,"Can't open title file %s.\n", buf);
		perror(Prog);
		prt_err(1);
		exit(1);
	}

	open_mast(mast_name);
}

prt_err(flg)
int	flg;
{
	char buf[BUFSIZ], *p;
	int		i;

	fprintf(ep,"Errno=%d, Doserrno=%d\n", errno, _doserrno);
	fprintf(ep,"keyfld is >>%s<<\n", keyfld);
	fprintf(ep,"fnum = %d, album = %d\n", fnum, album);
	fprintf(ep,"Index location is %ld, Data location is %ld.\n",
						ndxloc, datloc);
	fprintf(ep,"Data buffer >>%s<<\n", datbuf);
	fprintf(ep,"Key fields are:\n");
	for (i=0; i < nflds; i++) fprintf(ep,"fld[%d]=>>%s<<\n", i, flds[i]);
	if (flg)
	{
		printf("\n Keep going? (y or n): ");
		gets(buf);
		*(p=clean_up(buf)) != 'y';
		if (*p) return 1; else return 0;
	}
}
