/*
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 *				**** TST FIND ****
 *
 * This file contains the test driver for find_key
 *
 */

#define	MAXMATCH	20

#include <stdio.h>
#include "cd.h"

main(argc,argv)
int	argc;
char	**argv;
{
	char	buf[BUFSIZ];			/* input buffer */
	char	datbuf[BUFSIZ];			/* data file buffer */
	char	mast_name[BUFSIZ];		/* master file name */
	char	data_name[BUFSIZ];		/* data file name */
	char	key_name[BUFSIZ];		/* key file name */

	FILE	*mast_file;				/* master input file */
	FILE	*key_file;				/* input key file */
	FILE	*ndxfile[MAXNDX];		/* index file pointers */
	FILE	*datfile[MAXNDX];		/* data file pointers */
	char	ndxname[BUFSIZ];		/* index file name */
	int		ndxmax;					/* number of data (index) files */
	char	*flds[MASTFLDS];		/* for splitting master record */
	int		nflds;					/* number of fields found */
	int		fnum;					/* file number */
	long	ndxloc;					/* seek location for index file */
	long	datloc;					/* seek location for data file */

	int		nmatches;
	struct	key_match matches[MAXMATCH];
	char	*p;
	char	*key;
	int		i;


	Prog = argv[0];

	while (1)
	{
		printf("Enter master file name: ");
		gets(mast_name);
		clean_up(mast_name);
		if (*mast_name == EOS) exit(0);

		if ((mast_file = fopen(mast_name,"r")) == NULL)
		{
			fprintf(stderr,"%s: Can't open master file %s\n", Prog, mast_name);
			continue;
		}

		printf("Enter key file name: ");
		gets(key_name);
		clean_up(key_name);
		if (*key_name == EOS) continue;
		if ((key_file = fopen(key_name,"r")) == NULL)
		{
			fprintf(stderr,"%s: Unable to open key file %s\n", Prog, key_name);
			exit(1);
		}

		ndxmax = 0;				/* no files found yet */
		while (fgets(buf,sizeof(buf),mast_file) != NULL)
		{		/* read names of data files */
			clean_up(buf);		/* clean up buffer */
			if (*buf == EOS) continue;
			if (ndxmax >= MAXNDX)
			{
				fprintf(stderr,"%s: More than %d index files in %s\n",
						Prog, MAXNDX, mast_name);
				exit(1);
			}

			ndxmax++;			/* number of files so far */

			if ((nflds = split_flds(buf,flds,'|',0,MASTFLDS)) != MASTFLDS)
			{		/* wrong number of fields */
				fprintf(stderr,"%s: %d fields in master record %d instead of %d\n",
						Prog, nflds, ndxmax, MASTFLDS);
				continue;
			}

			strcpy(ndxname,flds[0]);		/* get data file name */
			for (p=ndxname; *p != EOS && *p != '.'; p++);
			strcpy(p,".ndx");
			if ((ndxfile[ndxmax] = fopen(ndxname,"r")) == NULL)
			{
				fprintf(stderr,"%s: Unable to open index file %s\n",
						Prog, ndxname);
				exit(1);
			}

			strcpy(p,".dat");
			if ((datfile[ndxmax] = fopen(ndxname,"r")) == NULL)
			{
				fprintf(stderr,"%s: Unable to open data file %s\n",
						Prog, ndxname);
				exit(2);
			}
		}		/* end loop thru data files in master file */


		*buf = '@';
		while (*buf != EOS)
		{
			printf("Enter string: ");
			gets(buf);
			clean_up(buf);
			if (*buf == EOS) continue;
			key = get_key(buf);
			nmatches = find_key(key_file,key,matches,MAXMATCH);
			printf("%d matches for key string >>%s<<\n", nmatches, key);
			for (i=0; i < nmatches; i++)
			{
				fnum = matches[i].km_file;
				printf("Match %d: File %d, record %ld\n",
					i, fnum, matches[i].km_rec);
				ndxloc = matches[i].km_rec;	/* record number */
				ndxloc *= sizeof(long);		/* displacement */
					/* position index file and read seek loc for data file */
				fseek(ndxfile[fnum],ndxloc,SEEK_SET);
				fread(&datloc,sizeof(datloc),1,ndxfile[fnum]);

					/* position data file and read album line */
				fseek(datfile[fnum],datloc,SEEK_SET);
				if (fgets(datbuf,sizeof(datbuf),datfile[fnum]) == NULL)
				{
					fprintf(stderr,"%s: Unable to seek to location %ld in file %d\n",
							Prog, datloc, fnum);
					exit(3);
				}
				clean_up(datbuf);
				puts(datbuf);
			}		/* end loop thru matches */
		}			/* user entered null string */
		for (i=0; i < ndxmax; i++)
		{		/* loop thru index/data files */
			fclose(ndxfile[i]);
			fclose(datfile[i]);
		}
	}				/* user entered null file name */
}
