/*
 *				***** WRITE TAPE ****
 *
 * (c) Copyright 1992 by William R. Pringle - all rights reserved
 *
 * EMACS_MODES: !fill, c, tabstop=4
 *
 * This file contains routines to write out tape configuration
 *	in a variety of formats
 *
 */

#include <stdio.h>
#include <time.h>
#include "cd.h"

write_tape()
{
	char	bname[BUFSIZ];				/* base file name */
	char	buf[BUFSIZ];				/* input buffer */
	char	*p;							/* temp. ptr */

	strcpy(bname,"tsave");				/* default dump file */
	printf("\nEnter base name (1 to 8 characters): ");
	gets(buf);
	p = clean_up(buf);					/* clean up base name */
	if (*p != EOS) strcpy(bname,p);		/* use whatever we were given */

	*buf = EOS;						/* make sure we start loop */
	while (*buf != 'q')
	{		/* loop for output formats */
		printf("\nEnter Output Format, or press ENTER for list (c,l,a,q,?): ");
		gets(buf);
		p = clean_up(buf);
		switch (*p)
		{
			case '?':
			case EOS:		/* ENTER */
				printf("\nOutput Formats are:\n");
				printf("\tC\tCut List\n");
				printf("\tL\tCassette Label\n");
				printf("\tA\tAlbum List\n");
				printf("\tQ\tQuit\n");
				break;

						/* Cut List */
			case 'c':
			case 'C':
				write_cut(bname);
				break;

						/* Cassette Label */
			case 'l':
			case 'L':
				write_label(bname);
				break;

						/* Album List */
			case 'a':
			case 'A':
				write_album(bname);
				break;

						/* Quit */
			case 'q':
			case 'Q':
				*buf = 'q';
				break;

			default:	/* bad input */
				printf("Invalid option (%s).  Try again\n", buf);
				break;
		}
	}		/* end loop for options */
}

write_cut(bname)
char	*bname;
{
	FILE 	*fp;					/* output file pointer */
	char	fname[BUFSIZ];			/* output file name */
	char	buf[BUFSIZ];			/* input buffer */
	int		i;						/* loop variable */

	if (strcmp(bname,"*"))
	{		/* real file used */
		strcpy(fname,bname);
		strcat(fname,".cut");
		if ((fp = open_out(bname,".cut",1)) == NULL) return;
	} else fp = stdout;

	list_albums(fp,0);				/* identify albums */

	for (i=0; i < MAXSIDES; i++)
	{		/* loop thru all tape sides */
		if (tapes[i].ts_num > 0) ontape(fp,i);
		if (fp == stdout)
		{
			printf("Press ENTER to continue: ");
			gets(buf);
		}
	}

	if (fp != stdout)
	{		/* not stdout - write trailer and close */
		cut_trail(fp);
		fclose(fp);
	}
}

write_album(bname)
char	*bname;
{
	FILE 	*fp;					/* output file pointer */
	char	fname[BUFSIZ];			/* output file name */
	char	buf[BUFSIZ];			/* input buffer */

	if (strcmp(bname,"*"))
	{		/* real file used */
		strcpy(fname,bname);
		strcat(fname,".alb");
		if ((fp = open_out(bname,".out",1)) == NULL) return;
	} else fp = stdout;

	list_albums(fp,1);				/* complete listing of albums */
	if (fp == stdout)
	{		/* output to stdout */
		printf("Press ENTER to continue: ");
		gets(buf);
	} else fclose(fp);
}

write_label(bname)
char	*bname;
{
	FILE 	*fp;					/* output file pointer */
	char	fname[BUFSIZ];			/* output file name */
	char	buf[BUFSIZ];			/* input buffer */
	char	*p;						/* tmp char ptr */
	char	obuf[BUFSIZ];			/* output buffer */
	int		Dbl;					/* double song flag */
	int		i;						/* loop variable */
	long	tsecs;					/* time variable */
	struct	tm	*tm;				/* time structure */

	if (strcmp(bname,"*"))
	{		/* real file used */
		strcpy(fname,bname);
		strcat(fname,".cas");
		if ((fp = open_out(bname,".cas",1)) == NULL) return;
	} else fp = stdout;

	for (i=0; i < MAXSIDES; i += 2)
	{		/* loop thru sides of tape */
		if (tapes[i].ts_num < 1) continue;

		printf("\nProcessing sides %d and %d ...\n", i+1, i+2);

			/* write header info */
		if (i > 0) fprintf(fp,"\nWrapUp\n");

		if ((i % 2)==0) fprintf(fp,"LeftSide\n");
		else fprintf(fp,"RightSide\n");

				/* date of recording */
		time(&tsecs);					/* get current time */
		tm = localtime(&tsecs);			/* convert to local time */
		sprintf(obuf,"%d/%d/%d", tm->tm_mon+1, tm->tm_mday,tm->tm_year+1900);

		printf("\nEnter date of recording (default:%s): ", obuf);
		gets(buf);
		p = clean_up(buf);				/* get clean response */
		if (*p != EOS) strcpy(obuf,p);	/* use what we got */
		fprintf(fp,"Date\t%s\n", obuf);			/* output date */

			/* noise reduction */
		strcpy(obuf,"dolby");
		printf("\nEnter noise reduction (default:%s): ", obuf);
		gets(buf);
		p = clean_up(buf);
		if (*p != EOS) strcpy(obuf,p);
		fprintf(fp,"Noise\t%s\n", obuf);

			/* artist */
		if (strcmp(tapes[i].ts_art,tapes[i+1].ts_art))
		{		/* different artists, concatenate both */
			fprintf(fp,"LeftArtist\t%s\n",tapes[i].ts_art);
			fprintf(fp,"RightArtist\t%s\n",tapes[i+1].ts_art);
		} else
		{		/* same artist, use full artist */
			fprintf(fp,"FullArtist\t%s\n",tapes[i].ts_art);
		}

			/* title */
		if (strcmp(tapes[i].ts_name,tapes[i+1].ts_name))
		{		/* different names */
			fprintf(fp,"LeftTitle\t%s\n",tapes[i].ts_name);
			fprintf(fp,"RightTitle\t%s\n",tapes[i+1].ts_name);
		} else
		{
			fprintf(fp,"FullTitle\t%s\n",tapes[i].ts_name);
		}

			/* large tab */

		printf("\nEnter text for left tab (or ENTER to skip): ");
		gets(buf);
		p = clean_up(buf);
		if (*p != EOS) fprintf(fp,"LeftTab\t%s\n", buf);

		printf("\nEnter text for right tab (or ENTER to skip): ");
		gets(buf);
		p = clean_up(buf);
		if (*p != EOS) fprintf(fp,"RightTab\t%s\n", buf);

		printf("\nEnter text for full tab (or ENTER for none): ");
		gets(buf);
		p = clean_up(buf);
		if (*p != EOS) fprintf(fp,"FullTab\t%s\n", buf);

		Dbl = '\0';		/* no double songs yet */
		printf("PrintSong options (Left, Right, Center, Double): ");
		gets(buf);
		p = clean_up(buf);
		if (*p == 'l' || *p == 'L') fprintf(fp,"LeftSongs\n");
		else if (*p == 'r' || *p == 'R') fprintf(fp,"RightSongs\n");
		else if (*p == 'c' || *p == 'C') fprintf(fp,"CenterSongs\n");
		else if (*p == 'd' || *p == 'D')
		{		/* double - artist or time */
			printf("Double - Enter Arist or Time (a or t): ");
			gets(buf);
			p = clean_up(buf);
			if (*p == 'a' || *p == 'A') Dbl = 'a';
			else Dbl = 't';
		} else printf("Invalid option (%s) -- ignored.\n", buf);

		if (tapes[i].ts_num > 0)
		{
			fprintf(fp,"\nLeftSide\n");
			write_tape_side(i,Dbl,fp);		/* print left side */
		}

		if (tapes[i+1].ts_num > 0)
		{
			fprintf(fp,"\nRightSide\n");
			write_tape_side(i+1,Dbl,fp);
		}
	}		/* end loop thru sides */
}

write_tape_side(i,Dbl,fp)
int		i;				/* what side to print */
int		Dbl;			/* double song flag */
FILE	*fp;			/* output file */
{
	int		j;			/* loop variable */

	/* now list all cuts taped */
	for (j=0; j < tapes[i].ts_num; j++)
	{		/* loop thru cuts on tape */
		if (Dbl == 'a')
		{
			fprintf(fp,"DoubleSong\t%s\t%s\n",
				tapes[i].ts_tape[j].t_cname,
				tapes[i].ts_tape[j].t_cart);
		} else if (Dbl == 't')
		{
			fprintf(fp,"DoubleSong\t%s\t%s\n",
				tapes[i].ts_tape[j].t_cname,
				str_time(tapes[i].ts_tape[j].tape_sec));
		} else
		{
			fprintf(fp,"\t%s\n", tapes[i].ts_tape[j].t_cname);
		}		/* end loop thru cuts for first side */
	}
}
